import React, { useEffect, useState } from "react";
import { Button, TextField, Dialog, DialogActions, DialogContent, DialogTitle, Typography, makeStyles } from "@material-ui/core";
import { Autocomplete, Alert } from "@material-ui/lab";
import { useCSEntrySearch, useTagsAndCommitIds } from "../../api/SwaggerApi";
import { useTypingTimer } from "../common/SearchBoxFilter/TypingTimer";

const useStyles = makeStyles((theme) => ({
  textField: {
    marginBottom: theme.spacing(1),
  },
  alert: {
    marginTop: theme.spacing(1),
  },
}));

export function IOCDeployDialog({ open, setOpen, submitCallback, hasActiveDeployment, init = {}, error, resetError}) {
  function onError(message) {
    console.log(message);
  }

  const classes = useStyles();
  const [hosts, getHosts] = useCSEntrySearch();
  const [host, setHost] = useState(null);
  const [query, onHostKeyUp] = useTypingTimer({interval: 500});
  const [tagOrCommitId, getTagOrCommitId] = useTagsAndCommitIds(onError);
  const [gitRepo, setGitRepo] = useState(init.git || "");
  const [gitVersion, setGitVersion] = useState(init.version || "");
  const gitProjectId = init.gitProjectId;

  const handleClose = () => {
    setOpen(false);
  };

  console.log({...init});

  useEffect(() => getHosts(`fqdn:"${query}"`), [query, getHosts, getTagOrCommitId]);

  const onSubmit = (event) => {
    event.preventDefault();
    const { comment: commentText, git: gitText } = event.currentTarget.elements;
    const comment = commentText.value;
    const git = gitText.value;

    submitCallback({  
      comment: comment,
      sourceUrl: git, 
      sourceVersion: gitVersion, 
      hostCSEntryId: host ? parseInt(host.csEntryHost.id) : (init.csEntryHost ? init.csEntryHost.id : undefined)});
  };

  return (
    <Dialog open={open} onClose={handleClose}>
      <form onSubmit={onSubmit}>
        <DialogTitle id="form-dialog-title">{ hasActiveDeployment ? "Deploy new revision" : "Deploy" }</DialogTitle>
        <DialogContent>
          <TextField autoComplete="off" className={classes.textField} id="comment" label="Deployment comment" variant="outlined" fullWidth/>
          <TextField autoComplete="off" className={classes.textField} id="git" label="Git repository" variant="standard" defaultValue={init.git || ""} fullWidth
            onChange={(event) => {setGitRepo(event.target.value); resetError();}} disabled required/>
          
          <Autocomplete
            className={classes.textField}
            autoHighlight
            id="version"
            options={tagOrCommitId}
            defaultValue = { init.version ? {
              "reference": init.version,
              "shortReference": init.shortVersion
            } : null }
            onFocus={(event) => {getTagOrCommitId(gitProjectId);}}
            getOptionLabel={(option) => {return option.shortReference}}
            onChange={(event, value, reason) => {setGitVersion(value?.reference); resetError();}}
            disabled={(!gitRepo) || (gitRepo.trim() === "")}
            renderInput={(params) => <TextField {...params} label="Git reference" variant="outlined" fullWidth required />}
          />

          <Autocomplete
            className={classes.textField}
            autoHighlight
            id="host"
            options={hosts.hostList}
            defaultValue = { init }
            getOptionLabel={option => {console.log(option); return option?.csEntryHost?.fqdn}}
            renderInput={(params) => <TextField {...params} label="host" variant="outlined" required/>}
            onChange={(event, value, reason) => {setHost(value); resetError();}}
            onInputChange={(event, value, reason) => {event && onHostKeyUp(event.nativeEvent)}}
            disabled={hasActiveDeployment}
          />

          { hasActiveDeployment ? 
          <Typography variant="body">
            Hint: First undeploy IOC if you want to move to another host.
          </Typography> 
          : <></>
          }

          {error ?
            <Alert severity="error"className={classes.alert}>{error}</Alert>
          : <></>}
        </DialogContent>
        <DialogActions>
          <Button onClick={handleClose} color="primary">Cancel</Button>
          <Button color="primary" variant="contained" type="submit" disabled={hasActiveDeployment && init?.version === gitVersion}>Deploy</Button>
        </DialogActions>
      </form>
    </Dialog>
  );
}