import { Button, Link as MuiLink, Tooltip, Typography } from "@material-ui/core";
import React, { useState, useEffect } from "react";
import { IOCDetails } from "../../components/IOC/IOCDetails";
import { DeployIOC } from "../../components/IOC/DeployIOC";
import { UndeployIOC } from "../../components/IOC/UndeployIOC";
import { SimpleModal } from "../../components/common/SimpleModal/SimpleModal";
import { useUpdateAndDeployIoc, useCreateUndeployment, useDeploymentListForIOC } from "../../api/SwaggerApi";
import GitRefLink from "./GitRefLink";
import AlertMessages from "./AlertMessages";
import { Link } from 'react-router-dom';
import { DeploymentAsyncList } from "../../components/deployments/DeploymentAsyncList";
import { SimpleAccordion } from "../common/Accordion/SimpleAccordion";
import { initRequestParams } from "../common/Helper";
import AccessControl from "../auth/AccessControl";

export function IOCManage({ ioc, getIOC, buttonDisabled }) {
  const [deployDialogOpen, setDeployDialogOpen] = useState(false);
  const [undeployDialogOpen, setUndeployDialogOpen] = useState(false);
  const [deployments, getDeployments] = useDeploymentListForIOC();
  const [deploymentList, setDeploymentList] = useState([]);

  const [lazyParams, setLazyParams] = useState({
    first: 0,
    rows: 5,
    page: 0
  });
  
  const [columnSort, setColumnSort] = useState({
    sortField: null,
    sortOrder: null
  });

  const rowsPerPage = [5, 10, 20, 50];

  useEffect(() => {
    setDeploymentList(deployments.deploymentList);
  }, [deployments, setDeploymentList])

  useEffect(() => 
    {
      if(ioc) {
        let requestParams = initRequestParams(lazyParams, null, columnSort);
        
        requestParams.iocId = ioc.id;
        
        if(columnSort.sortField === "start") {
          requestParams.orderBy = "START_TIME";
        }
      
        if(columnSort.sortField === "user") {
          requestParams.orderBy = "CREATED_BY";
        }
      
        if(columnSort.sortField === "ioc") {
          requestParams.orderBy = "IOC_NAME";
        }
      
        if(columnSort.sortField === "version") {
          requestParams.orderBy = "GIT_REFERENCE";
        }

        getDeployments(requestParams);
      }
    }, [ioc, lazyParams, columnSort, getDeployments]);

  const closeDeployModal = () => {
    setDeployDialogOpen(false);
    getIOC();
    // history.push(window.location.pathname);
  }

  const closeUndeployModal = () => {
    setUndeployDialogOpen(false);
  }

  let content = <></>;
  if (ioc) {
    console.log({ ...ioc });
    const managedIOC = { ...ioc };

    const getSubset = (ioc) => {
      ioc = { ...ioc };
      const {
        description,
        // active, 
      } = ioc;
      const { sourceUrl: git } = ioc;
      let host = ioc.activeDeployment?.host.fqdn;

      //create link if host is set
      if (ioc.activeDeployment) {
        host = 
        <Typography>
          {ioc.activeDeployment.host.csEntryId ?
            <Link to={`/hosts/${ioc.activeDeployment.host.csEntryId}`}>{ioc.activeDeployment.host.fqdn}</Link>
            :
            <>{ioc.activeDeployment.host.fqdn}</>
          }
        </Typography>
      }

      return {
        "CCDB name": <Typography><MuiLink href={`${window.CCDB_ADDRESS}?name=${ioc.namingName}`} 
                        target="_blank" rel="noreferrer">{ioc.namingName}</MuiLink></Typography>,
        description,
        "Owner": ioc.owner,
        git: <Typography><MuiLink href={git} target="_blank" rel="noreferrer">{git}</MuiLink></Typography>,
        "Git reference" : ioc.sourceVersion ? <GitRefLink ioc={ioc} /> : '---',
        "target host": host || '---',
      };
    };

    const formInit = {
      name: ioc.namingName,
      description: ioc.description,
      version: ioc.sourceVersion,
      shortVersion: ioc.sourceVersionShort,
      git: ioc.sourceUrl,
      gitProjectId: ioc.gitProjectId
    };

    if (ioc.activeDeployment) {
      formInit.csEntryHost = {
        fqdn: ioc.activeDeployment.host.fqdn,
        id: ioc.activeDeployment.host.csEntryId,
      }
    }

    let disabledDebployButtonTitle = "";
    if (buttonDisabled || ioc.operationInProgress) {
      disabledDebployButtonTitle = "There is an ongoing operation, you can not deploy the IOC right now";
    }

    let disabledUndeployButtonTitle = "";
    if (buttonDisabled || ioc.operationInProgress) {
      disabledUndeployButtonTitle = "There is an ongoing operation, you can not undeploy the IOC right now";
    }

    content = (
      <>
        <IOCDetails ioc={managedIOC} getSubset={getSubset} alert={<AlertMessages alerts={managedIOC.alerts}/>}
          buttons={
            <>
              <Tooltip title={disabledDebployButtonTitle}>
                <span>
                  <Button variant="contained" color="secondary" onClick={() => { setDeployDialogOpen(true) }} disabled={buttonDisabled || ioc.operationInProgress}>
                    {ioc.activeDeployment ? "Deploy new revision" : "Deploy"}
                  </Button>
              </span>
              </Tooltip>
              {ioc.activeDeployment ? 
                <Tooltip title={disabledUndeployButtonTitle}>
                  <span>
                    <Button variant="contained" color="secondary" style={{ marginRight: 20 }} onClick={() => { setUndeployDialogOpen(true) }} 
                      disabled={buttonDisabled || ioc.operationInProgress}>Undeploy
                    </Button>
                  </span>
                </Tooltip> :
                <></> 
              }
            </>
          } 
        />
        <AccessControl allowedRoles={["DeploymentToolAdmin", "DeploymentToolIntegrator"]}
          renderNoAccess={() => <></>}>
          <SimpleAccordion summary="Deployment log">
            <DeploymentAsyncList deployments={deploymentList} setDeployments={setDeploymentList} totalCount={deployments.totalCount}
              lazyParams={lazyParams} setLazyParams={setLazyParams} columnSort={columnSort} setColumnSort={setColumnSort}
              rowsPerPage={rowsPerPage} />
          </SimpleAccordion>
          <SimpleModal open={deployDialogOpen} setOpen={setDeployDialogOpen}>
            <DeployIOC open={deployDialogOpen} setOpen={setDeployDialogOpen} submitCallback={closeDeployModal} init={formInit} hook={useUpdateAndDeployIoc.bind(null, ioc.id)}
              hasActiveDeployment={ioc.activeDeployment} />
          </SimpleModal>
          <SimpleModal open={undeployDialogOpen} setOpen={setUndeployDialogOpen}>
            <UndeployIOC open={undeployDialogOpen} setOpen={setUndeployDialogOpen} submitCallback={closeUndeployModal} ioc={ioc} hook={useCreateUndeployment.bind(null, ioc.id)} />
          </SimpleModal>
        </AccessControl>
      </>
    );
  }

  return (
    <>
      {content}
    </>
  );

}