import React, { useCallback } from 'react';
import { CustomTable } from "../common/table/CustomTable";
import { Grid, Tooltip } from "@material-ui/core";
import { useHistory } from 'react-router-dom';
import { IOCStatusIcon } from './IOCIcons';

const ownIocsColumns = [
  { id: "status", label: 'Status', width: '8%' },
  { id: "namingName", label: 'IOC name', width: '20%', sortable: false },
  { id: "host", label: 'Host', width: '15%', sortable: false },
  { id: "network", label: 'Network', width: '15%', sortable: false },
  { id: "sourceVersion", label: 'Git reference', width: '42%', sortable: false },
]

const exploreIocsColumns = [
  { id: "status", label: 'Status', width: '8%' },
  { id: "namingName", label: 'IOC name', width: '20%', sortable: true },
  { id: "host", label: 'Host', width: '15%', sortable: false },
  { id: "network", label: 'Network', width: '15%', sortable: false },
  { id: "owner", label: 'Owner', width: '10%', sortable: true },
  { id: "sourceVersion", label: 'Git reference', width: '32%', sortable: true },
]

const hostDetailsColumns = [
  { id: "status", label: 'Status', width: '8%' },
  { id: "name", label: 'Name', width: '30%', sortable: false },
  { id: "sourceVersion", label: 'Git reference', width: '62%', sortable: false },
]

function createGitVersionField(version, shortVersion) {
  return(
    <>
      {version ?
        <Tooltip title={version} arrow enterDelay={400}>
          <label>{shortVersion ?? version}</label>
        </Tooltip>
        : "---"
      }
    </>
  )
}

function createTableRowForHostDetails(ioc) {
  const { id, name, namingName, sourceVersion, sourceVersionShort, alert } = ioc;

  return {
    id: id,
    status: 
      <Grid container direction="column" justifyContent="center" alignItems="center">
        <IOCStatusIcon ioc={ioc} />
      </Grid>,
    name: namingName ?? name,
    sourceVersion: createGitVersionField(sourceVersion, sourceVersionShort),
    discrepancy: alert === "WARNING",
    inconsistentState: alert === "ERROR",
  };
}

function createTableRowForOwnIocs(ioc) {
  const { id, activeDeployment, namingName, sourceVersion, sourceVersionShort, alert} = ioc;

  return {
    id: id,
    status: 
    <Grid container direction="column" justifyContent="center" alignItems="center">
      <IOCStatusIcon ioc={ioc} />
    </Grid>,
    namingName: namingName ?? "---",
    sourceVersion: createGitVersionField(sourceVersion, sourceVersionShort),
    host: (activeDeployment?.host.hostName || "---"),
    network: (activeDeployment?.host.network || "---"),
    discrepancy: alert === "WARNING",
    inconsistentState: alert === "ERROR",
    iocNotDeployed: !ioc.activeDeployment
  };
}

function createTableRowForExploreIocs(ioc) {
  const { id, namingName, owner, sourceVersion, sourceVersionShort, alert, host} = ioc;

  return {
    id: id,
    status: 
    <Grid container direction="column" justifyContent="center" alignItems="center">
      <IOCStatusIcon ioc={ioc} />
    </Grid>,
    namingName: namingName ?? "---",
    sourceVersion: createGitVersionField(sourceVersion, sourceVersionShort),
    host: (host?.hostName || "---"),
    network: (host?.network || "---"),
    owner: (owner || "---"),
    discrepancy: alert === "WARNING",
    inconsistentState: alert === "ERROR",
    iocNotDeployed: !ioc.activeDeployment
  };
}

export function IOCTable({ iocs, rowType="own", totalCount, lazyParams, setLazyParams, columnSort, setColumnSort, rowsPerPage, loading, paginator}) {
  const tableTypeSpecifics = {
    "own": [ownIocsColumns, createTableRowForOwnIocs],
    "host": [hostDetailsColumns, createTableRowForHostDetails],
    "explore" : [exploreIocsColumns, createTableRowForExploreIocs]
  }

  const history = useHistory();
  const [columns, createRow] = tableTypeSpecifics[rowType];
  const rows = iocs.map(ioc => createRow(ioc));

  const onRowClicked = useCallback((id) => {
    history.push(`/iocs/${id}`);
  }, [history]);

  return (
    <CustomTable 
      columns={columns} 
      totalCount={totalCount} 
      rows={rows} 
      handleRowClick={onRowClicked} 
      lazyParams={lazyParams} 
      setLazyParams={setLazyParams} 
      columnSort={columnSort} 
      setColumnSort={setColumnSort} 
      rowsPerPage={rowsPerPage} 
      loading={loading}
      paginator={paginator}
    />
  );
}
