import React, { createContext, useContext, useEffect, useState } from "react";
import {
  AppBar,
  Avatar,
  Button,
  IconButton,
  ListItem,
  ListItemIcon,
  ListItemText,
  makeStyles,
  Toolbar,
  Typography,
  Menu,
  MenuItem,
  Tooltip,
  Badge,
  Popover,
  Grid,
  List,
  Box,
  Divider,
  useTheme
} from "@material-ui/core";
import { withStyles } from '@material-ui/core/styles';
import { LockOpen, Home, Assignment, Storage, TrendingUp } from "@material-ui/icons";
import HelpIcon from '@material-ui/icons/Help';
import InfoIcon from '@material-ui/icons/Info';
import { DeploymentStatusIcon } from "../../deployments/DeploymentIcons";
import DeleteIcon from '@material-ui/icons/Delete';
import NotificationsIcon from '@material-ui/icons/Notifications';
import { MenuDrawer } from "../Menu/MenuDrawer";
import { Link } from "react-router-dom";
import { userContext } from "../../../api/SwaggerApi";
import { notificationContext } from "../../common/notification/Notifications";
import { useWindowDimensions } from "../../common/Helper";
import { CCCEControlSymbol } from '../../../icons/CCCEControlSymbol';
import Login from "../../auth/Login";
import { SimpleModal } from "../../common/SimpleModal/SimpleModal";
import AccessControl from "../../auth/AccessControl";
import { useHistory } from "react-router-dom";
import { applicationSubTitle } from "../../common/Helper";

const useStyles = makeStyles((theme) => ({
  root: {
    flexGrow: 1,
    paddingBottom: theme.spacing(2),
  },
  appBar: {
    zIndex: 1250 // fixme: a hack to make the appBar stay on top of the drawer, should be between 1201 - 1299
  },
  menuButton: {
    marginRight: theme.spacing(2),
  },
  helpDivider: {
    marginTop: theme.spacing(5),
  },
  title: {
    flexGrow: 1,
  },
  menuIcon: {
    marginLeft: theme.spacing(1),
  },
  typography: {
    padding: theme.spacing(2),
  },
  notificationButton: {
    marginRight: theme.spacing(2),
  },
  content: {
    marginTop: theme.spacing(1),
  },
  notificationList: {
    overflow: 'auto',
    maxHeight: 360,
  },
  shiftContentLeft: {
    marginLeft: theme.drawer.widthClose,
  },
  shiftContentRight: {
    marginLeft: theme.drawer.widthOpen,
  },
}));

const StyledMenu = withStyles({
  paper: {
    border: '1px solid #d3d4d5',
  },
})((props) => (
  <>
    <div style={{ ...props.theme.mixins.toolbar }} />
    <Menu
      elevation={0}
      getContentAnchorEl={null}
      anchorOrigin={{
        vertical: 'bottom',
        horizontal: 'center',
      }}
      transformOrigin={{
        vertical: 'top',
        horizontal: 'center',
      }}
      {...props}
    />
  </>
));

const StyledMenuItem = withStyles((theme) => ({
  root: {
    '&:focus': {
      backgroundColor: theme.palette.common.gray,
      '& .MuiListItemIcon-root, & .MuiListItemText-primary': {
        color: theme.palette.common.black,
      },
    },
  },
}))(MenuItem);

export function ButtonAppBar({ homeUrl, homeClick, title, button, profileButton, notificationsButton }) {
  const classes = useStyles();

  return (
    <div className={classes.root}>
      <AppBar position="fixed" className={classes.appBar}>
        <Toolbar>
          <IconButton edge="start" component={Link} to={homeUrl} onClick={homeClick} className={classes.menuButton} color="inherit" aria-label="menu">
            <Home />
          </IconButton>
          <Typography variant="h1" className={classes.title} noWrap>
            {title}
          </Typography>
          {button}
          {notificationsButton}
          {profileButton}
        </Toolbar>
      </AppBar>
    </div>
  );
}

const defaultTitle = "CCCE"
export const GlobalAppBarContext = createContext({
  title: defaultTitle,
  setButton: () => { console.log("default GlobalAppBarContext.setButton does nothing.") },
  setTitle: () => { console.log("default GlobalAppBarContext.setTitle does nothing.") }
});

function MenuListItem({ url, icon, text, tooltip }) {
  const classes = useStyles();
  const currentUrl = `${window.location}`;
  console.log(currentUrl);
  return (
    <>
      {url.includes("about") ? <Divider className={classes.helpDivider} /> : <></>}
      <Tooltip title={tooltip ? text : ""} placement="right" arrow>
        <ListItem button component={Link} to={url} selected={currentUrl.endsWith(url)}>
          <ListItemIcon className={classes.menuIcon}>{icon}</ListItemIcon>
          <ListItemText primary={text} primaryTypographyProps={{ variant: "h5" }} />
        </ListItem>
      </Tooltip>
    </>
  );
}

function MenuListItems({ menuItems, drawerOpen }) {
  return (
    <>
      {menuItems.map(({ text, url, icon }) => (
        <MenuListItem key={text} tooltip={!drawerOpen}  url={url} icon={icon} text={text} />
      ))}
    </>
  );
}

export function UserInfo({ user }) {
  console.log(user);
  return (
    <ListItem>
      <ListItemIcon>
        <Avatar src={user.avatar} variant="rounded" />
      </ListItemIcon>
      <ListItemText primary={user.fullName} secondary={user.email} />
    </ListItem>
  );
}

export function ProfileMenu() {
  const theme = useTheme();
  const { user, logout, login, loginError, resetLoginError } = useContext(userContext);
  const [anchorEl, setAnchorEl] = useState(null);
  const [loginFormOpen, setLoginFormOpen] = useState(false);
  const history = useHistory();

  const handleClick = (event) => {
    setAnchorEl(event.currentTarget);
  };

  const handleClose = () => {
    setAnchorEl(null);
  };

  const goHome = () =>{ 
    history.push("/home");
  }

  useEffect(() => {
    if (user) {
      setLoginFormOpen(false);
    }
  }, [user, setLoginFormOpen]);

  return (
    <>
    { user ?
      <>
      <Avatar src={user.avatar} variant="circular" onClick={handleClick} aria-controls="profile-menu"
        aria-haspopup="true" />
      <StyledMenu
        id="profile-menu"
        anchorEl={anchorEl}
        keepMounted
        open={Boolean(anchorEl)}
        onClose={handleClose}
        theme={theme}
      >
        <StyledMenuItem onClick={() => { handleClose(); logout(); goHome(); }}>
          <ListItemIcon>
            <LockOpen fontSize="small" />
          </ListItemIcon>
          <ListItemText primary="Logout" primaryTypographyProps={{ variant: "h5" }} />
        </StyledMenuItem>
      </StyledMenu>
    </>
    :
    <>
      <Button color="inherit" onClick={() => { resetLoginError(); setLoginFormOpen(true) }}>Login</Button>
      <SimpleModal open={loginFormOpen} setOpen={setLoginFormOpen}>
        <Login open={loginFormOpen} setOpen={setLoginFormOpen} auth={login} error={loginError} resetError={resetLoginError}/>
      </SimpleModal>
    </>
  }
  </>
  );
}

const StyledListItem = withStyles(() => ({
  root: {
    margin: 0,
    paddingLeft: 10,
  },
}))(ListItem);

const StyledListItemIcon = withStyles(() => ({
  root: {
    minWidth: 0,
    marginRight: 12,
  },
}))(ListItemIcon);

export function NotificationMenu() {
  const [anchorEl, setAnchorEl] = useState(null);
  const { user } = useContext(userContext);
  const classes = useStyles();
  const { notifications, clearNotifications } = useContext(notificationContext);

  const handleClick = (event) => {
    setAnchorEl(event.currentTarget);
  };

  const handleClose = () => {
    setAnchorEl(null);
  };

  function haveNotification() {
    return (
      <>
        <Grid item xs={12}>
          <List className={classes.notificationList}>
            {notifications.map((notification) => {
              return (
                <>
                {notification.link 
                ?
                <StyledListItem button component={Link} to={notification.link} onClick={() => { handleClose() }}>
                  <StyledListItemIcon className={classes.notificationItem}>
                    <DeploymentStatusIcon status={notification.status}/>
                  </StyledListItemIcon>
                  <Typography className={classes.notificationItem}>{notification.message}</Typography>
                </StyledListItem>
                :
                <StyledListItem key={notification.message}>
                  <StyledListItemIcon className={classes.notificationItem}>
                    <DeploymentStatusIcon status={notification.status}/>
                  </StyledListItemIcon>
                  <Typography className={classes.notificationItem}>{notification.message}</Typography>
                </StyledListItem>
              }
              </>
              );
            })}
          </List>
        </Grid>
        <Grid item xs={1}>
          <Box display="flex" justifyContent="flex-end">
            <IconButton color="inherit" onClick={() => { clearNotifications(); handleClose(); }}>
              <Tooltip title="Clear all notifications" arrow>
                <DeleteIcon fontSize="small"/>
              </Tooltip>
            </IconButton>
          </Box>
        </Grid>
      </>
    )
  }

  function noNotification() {
    return <Typography inset="true" style={{ margin: '4px' }}>You have no notifications!</Typography>;
  }

  return (
    <>
    { user 
      ?
      <>
        <IconButton color="inherit" onClick={handleClick} className={classes.notificationButton}>
          <Badge badgeContent={notifications.length} color="secondary">
            <NotificationsIcon />
          </Badge>
        </IconButton>
        <Popover
          open={Boolean(anchorEl)}
          anchorEl={anchorEl}
          onClose={handleClose}
          anchorOrigin={{
            vertical: 'bottom',
            horizontal: 'center',
          }}
          transformOrigin={{
            vertical: 'top',
            horizontal: 'center',
          }}
        >
          <Grid container spacing={0} justify="flex-end">
            {(notifications.length > 0) ? haveNotification() : noNotification()}
          </Grid>
        </Popover>
      </>
      :
      <></>
    }
    </>
  );
}

export const DRAWER_OPEN = "DRAWER_OPEN";

export function GlobalAppBar({ children }) {
  const theme = useTheme();
  const classes = useStyles();
  const [button, setButton] = useState();
  const [title, setTitle] = useState(defaultTitle);
  const [value] = useState({ setButton, setTitle });
  const { width } = useWindowDimensions();
  const [drawerOpen, setDrawerOpen] = useState(isDrawerOpen() ?? width > 1730);

  useEffect(() => {
    if(isDrawerOpen()) {
      setDrawerOpen(width > 1730);
    }
  }, [width, setDrawerOpen]);

  const toggleDrawer = () => {
    storeDrawerOpen(!drawerOpen);
    setDrawerOpen(!drawerOpen);
  }

  function isDrawerOpen() {
    return JSON.parse(localStorage.getItem(DRAWER_OPEN));
  }

  function storeDrawerOpen(open) {
    localStorage.setItem(DRAWER_OPEN, JSON.stringify(open));
  }

  const profile = ProfileMenu();
  const notifications = NotificationMenu();

  const makeLink = (text, url, icon) => ({ text, url, icon });
  const menuItemsAll = [
    makeLink("Explore IOCs", "/iocs", <CCCEControlSymbol />),
    makeLink("Explore IOC hosts", "/hosts", <Storage />),
    makeLink("Deployment log", "/deployments", <Assignment />),
    makeLink("Statistics", "/statistics", <TrendingUp />),
    makeLink("About", "/about", <InfoIcon />),
    makeLink("Help", "/help", <HelpIcon />)
  ]

  const menuItemsVisitor = [
    makeLink("Explore IOCs", "/iocs", <CCCEControlSymbol />),
    makeLink("Explore IOC hosts", "/hosts", <Storage />),
    makeLink("Deployment log", "/deployments", <Assignment />),
    makeLink("Statistics", "/statistics", <TrendingUp />),
    makeLink("About", "/about", <InfoIcon />)
  ]

  return (
    <>
      <ButtonAppBar homeUrl="/home" title={title} button={button} profileButton={profile} notificationsButton={notifications} />
      <div>
      <MenuDrawer open={drawerOpen} toggleDrawer={toggleDrawer}>
        <AccessControl allowedRoles={["DeploymentToolAdmin", "DeploymentToolIntegrator"]} 
          renderNoAccess={() =>
            <MenuListItems menuItems={menuItemsVisitor}
               drawerOpen={drawerOpen} />
          }>
          <MenuListItems menuItems={menuItemsAll}
             drawerOpen={drawerOpen} />
        </AccessControl>
      </MenuDrawer>
      </div>
      <div>
        <GlobalAppBarContext.Provider value={value}>
          <div style={{ ...theme.mixins.toolbar }} />
          <div className={classes.content}>
            <div className={drawerOpen ? classes.shiftContentRight : classes.shiftContentLeft}>
              {children}
            </div>
          </div>
        </GlobalAppBarContext.Provider>
      </div>
    </>
  );
}

export function useGlobalAppBar(initTitle, initButton) {
  const [button, setButton] = useState(initButton);
  const [title, setTitle] = useState(initTitle);
  const appBar = useContext(GlobalAppBarContext);
  useEffect(() => {
    appBar.setButton(button);
    appBar.setTitle('CCCE ' + applicationSubTitle() + ' / ' + title );
  }, [appBar, title, button]);
  return { setButton, setTitle };
}